; This examples demonstrates how libusb's drivers
; can be installed automatically along with your application using an installer.
;
; Requirements: Inno Setup (http://www.jrsoftware.org/isdl.php)
;
; To use this script, do the following:
; - copy libusb's driver (libusb0.sys, libusb0.dll) to this folder
; - create an .inf and .cab file using libusb's 'inf-wiward.exe'
;   and save the generated files in this folder.
; - in this script replace <your_inf_file.inf> with the name of your .inf file
; - customize other settings (strings)
; - open this script with Inno Setup
; - compile and run

[Setup]
OutputDir=.
AppName=sTokenSoft
AppVerName=sTokenSoft 1.3-0
AppPublisher=CMM Sigma
AppPublisherURL=http://www.stoken.eu/soft
AppVersion=1.3
OutputBaseFilename=stokensoft_1.3-0
LicenseFile=license.gpl3.txt
DefaultDirName={pf}\sTokenSoft
DefaultGroupName=sTokenSoft
Compression=lzma
SolidCompression=yes
; Win2000 or higher
; MinVersion=5,5
PrivilegesRequired=admin
SetupIconFile=img\stokensoft_setup_icon.ico
WizardImageFile=img\stoken_install_164_314.bmp
WizardSmallImageFile=img\stokensoft_install_56_45.bmp

[Types]
Name: user; Description: Instalacja uytkownika
Name: admin; Description: Instalacja administratora
Name: custom; Description: Wybr skadnikw; Flags: iscustom

[Components]
Name: basic; Description: Podstawowe pakiety; Types: user admin custom; Flags: fixed
Name: stoken_softadmin; Description: sToken SoftAdmin; Types: user admin
Name: stoken_softvpn; Description: sToken SoftVPN; Types: user admin
Name: stoken_softca; Description: sToken SoftCA; Types: admin
Name: service; Description: OpenVPN Service; Types: custom

[Files]
; copy drivers to the App\drivers folder
Source: drivers\tap0901\tap0901.sys; DestDir: {app}\drivers\tap; Components: basic
Source: drivers\tap0901\tap0901.cat; DestDir: {app}\drivers\tap; Components: basic
Source: drivers\tap0901\OemWin2k.inf; DestDir: {app}\drivers\tap; Components: basic
Source: drivers\tap0901\tapinstall.exe; DestDir: {app}\drivers\tap; Components: basic

; copy Qt DLLs
Source: lib\libusb0.dll; DestDir: {app}\bin; Components: basic; FLags: ignoreversion
Source: lib\mingwm10.dll; DestDir: {app}\bin; Components: basic; FLags: ignoreversion
Source: lib\QtCore4.dll; DestDir: {app}\bin; Components: basic; FLags: ignoreversion
Source: lib\QtGui4.dll; DestDir: {app}\bin; Components: basic; FLags: ignoreversion
Source: lib\QtNetwork4.dll; DestDir: {app}\bin; Components: basic; FLags: ignoreversion
Source: lib\QtXml4.dll; DestDir: {app}\bin; Components: basic; FLags: ignoreversion

; copy applications to the App folder
Source: ..\..\bin\stoken_openssl.exe; DestDir: {app}\bin; Components: basic
Source: ..\..\bin\stoken_openvpn.exe; DestDir: {app}\bin; Components: basic
Source: ..\..\bin\stoken_softadmin.exe; DestDir: {app}\bin; Components: stoken_softadmin
Source: ..\..\bin\stoken_softvpn.exe; DestDir: {app}\bin; Components: stoken_softvpn
Source: ..\..\bin\stoken_softca.exe; DestDir: {app}\bin; Components: stoken_softca

; copy openvpn service and its components to the service folder
Source: ..\..\bin\openvpnserv.exe; DestDir: {app}\service; Components: service
Source: service\secp256r1.pem; DestDir: {app}\service; Components: service
Source: service\secp384r1.pem; DestDir: {app}\service; Components: service
Source: service\secp521r1.pem; DestDir: {app}\service; Components: service
Source: service\openvpn-ecc.ovpn.example.txt; DestDir: {app}\service; Components: service

; license file
Source: license.gpl3.txt; DestDir: {app}\license; Components: basic;

[Icons]
Name: {group}\sToken SoftAdmin; Filename: {app}\bin\stoken_softadmin.exe; Flags: createonlyiffileexists
Name: {commondesktop}\sToken SoftAdmin; Filename: {app}\bin\stoken_softadmin.exe; Flags: createonlyiffileexists
Name: {group}\sToken SoftVPN; Filename: {app}\bin\stoken_softvpn.exe; Flags: createonlyiffileexists
Name: {commondesktop}\sToken SoftVPN; Filename: {app}\bin\stoken_softvpn.exe; Flags: createonlyiffileexists
Name: {group}\sToken SoftCA; Filename: {app}\bin\stoken_softca.exe; Flags: createonlyiffileexists
Name: {commondesktop}\sToken SoftCA; Filename: {app}\bin\stoken_softca.exe; Flags: createonlyiffileexists
Name: {group}\Uninstall\Uninstall sToken; Filename: {uninstallexe}

[Languages]
Name: en; MessagesFile: compiler:Default.isl
Name: pl; MessagesFile: compiler:Languages\Polish.isl

[Registry]
Root: HKCU; Subkey: Software\CMM Sigma; Flags: uninsdeletekeyifempty
Root: HKCU; Subkey: Software\CMM Sigma\sTokenSoft; Flags: uninsdeletekey
Root: HKLM; Subkey: Software\CMM Sigma; Flags: uninsdeletekeyifempty
Root: HKLM; Subkey: Software\CMM Sigma\sTokenSoft; Flags: uninsdeletekey
Root: HKLM; Subkey: Software\CMM Sigma\sTokenSoft\Installer; ValueType: dword; ValueName: Version; ValueData: $01030000
Root: HKLM; Subkey: Software\CMM Sigma\sTokenSoft\Installer; ValueType: string; ValueName: Path; ValueData: {app}
Root: HKLM; Subkey: Software\CMM Sigma\sTokenSoft\Drivers\TAP; ValueType: dword; ValueName: Version; ValueData: $00010000
Root: HKLM; Subkey: Software\CMM Sigma\sTokenSoft\Drivers\TAP; ValueType: string; ValueName: Path; ValueData: {app}\drivers\tap

Root: HKLM; Subkey: Software\OpenVPN; Flags: uninsdeletekeyifempty
Root: HKLM; Subkey: Software\OpenVPN; Flags: uninsdeletekey
Root: HKLM; Subkey: Software\OpenVPN; ValueType: string; ValueName: exe_path; ValueData: {app}\bin\stoken_openvpn.exe
Root: HKLM; Subkey: Software\OpenVPN; ValueType: string; ValueName: config_dir; ValueData: {app}\service
Root: HKLM; Subkey: Software\OpenVPN; ValueType: string; ValueName: config_ext; ValueData: ovpn
Root: HKLM; Subkey: Software\OpenVPN; ValueType: string; ValueName: log_dir; ValueData: {app}\service
Root: HKLM; Subkey: Software\OpenVPN; ValueType: string; ValueName: priority; ValueData: NORMAL_PRIORITY_CLASS
Root: HKLM; Subkey: Software\OpenVPN; ValueType: string; ValueName: log_append; ValueData: 0

[Run]
; install TAP driver
Filename: {app}\drivers\tap\tapinstall.exe; WorkingDir: {app}\drivers\tap; Parameters: install OemWin2k.inf tap0901; StatusMsg: Instalowanie sterownika TAP (moe potrwa kilka sekund) ...; Flags: runhidden
; setup openvpn service
Filename: {app}\service\openvpnserv.exe; WorkingDir: {app}\service; Parameters: -remove; StatusMsg: Instalowanie usugi OpenVPN (moe potrwa kilka sekund) ...; Flags: runhidden skipifdoesntexist
Filename: {app}\service\openvpnserv.exe; WorkingDir: {app}\service; Parameters: -install; StatusMsg: Instalowanie usugi OpenVPN (moe potrwa kilka sekund) ...; Flags: runhidden skipifdoesntexist

[UninstallDelete]
Type: filesandordirs; Name: {app}

[UninstallRun]
Filename: {app}\drivers\tap\tapinstall.exe; Parameters: remove tap0901; StatusMsg: Deinstalowanie sterownika TAP (moe potrwa kilka sekund) ...; Flags: runhidden
Filename: {app}\service\openvpnserv.exe; WorkingDir: {app}\service; Parameters: -remove; StatusMsg: Deinstalowanie usugi OpenVPN (moe potrwa kilka sekund) ...; Flags: runhidden skipifdoesntexist

