/*
 *  Copyright (C) 2007-2008 CMM Sigma Andrzej Chmielowiec <cmmsigma@cmmsigma.eu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#include "X509SubjectDataWidget.h"

X509SubjectDataWidget::X509SubjectDataWidget(QWidget *parent): QWidget(parent)
{
	mCountryCodeValidator = new QRegExpValidator(QRegExp(QString("[A-Z]{2,2}")), this);
	mReqDataValidator = new QRegExpValidator(QRegExp(QString("[a-zA-Z0-9_\x20\x2d]{1,512}")), this);
	
	mCountryCode = new QLineEdit;
	mCountryCode->setValidator(mCountryCodeValidator);
	mStateOrProvinceName = new QLineEdit;
	mStateOrProvinceName->setValidator(mReqDataValidator);
	mLocalityName = new QLineEdit;
	mLocalityName->setValidator(mReqDataValidator);
	mOrganizationName = new QLineEdit;
	mOrganizationName->setValidator(mReqDataValidator);
	mOrganizationalUnitName = new QLineEdit;
	mOrganizationalUnitName->setValidator(mReqDataValidator);
	mCommonName = new QLineEdit;
	mCommonName->setValidator(mReqDataValidator);
	mEmailAddress = new QLineEdit;
	
	QGridLayout *layout = new QGridLayout;
	layout->addWidget(new QLabel(tr("Country code")), 0, 0);
	layout->addWidget(mCountryCode, 0, 1);
	layout->addWidget(new QLabel(tr("Region")), 1, 0);
	layout->addWidget(mStateOrProvinceName, 1, 1);
	layout->addWidget(new QLabel(tr("City")), 2, 0);
	layout->addWidget(mLocalityName, 2, 1);
	layout->addWidget(new QLabel(tr("Organization")), 3, 0);
	layout->addWidget(mOrganizationName, 3, 1);
	layout->addWidget(new QLabel(tr("Department")), 4, 0);
	layout->addWidget(mOrganizationalUnitName, 4, 1);
	layout->addWidget(new QLabel(tr("Common name<sup>*</sup>")), 5, 0);
	layout->addWidget(mCommonName, 5, 1);
	layout->addWidget(new QLabel(tr("E-mail")), 6, 0);
	layout->addWidget(mEmailAddress, 6, 1);
	layout->setRowStretch(7, 1);
	layout->setColumnStretch(1, 1);
	setLayout(layout);
}

QString
X509SubjectDataWidget::subjectOpenSSL()
{
	QString subject;
	
	if (mStateOrProvinceName->text() != QString())
		subject += QString("/C=").append(mCountryCode->text());
	
	if (mStateOrProvinceName->text() != QString())
		subject += QString("/ST=").append(mStateOrProvinceName->text());
	
	if (mLocalityName->text() != QString())
		subject += QString("/L=").append(mLocalityName->text());
	
	if (mOrganizationName->text() != QString())
		subject += QString("/O=").append(mOrganizationName->text());
	
	if (mOrganizationalUnitName->text() != QString())
		subject += QString("/OU=").append(mOrganizationalUnitName->text());
	
	if (mCommonName->text() != QString())
		subject += QString("/CN=").append(mCommonName->text());
	
	if (mEmailAddress->text() != QString())
		subject += QString("/emailAddress=").append(mEmailAddress->text());
	
	return subject;
}

bool
X509SubjectDataWidget::isReady()
{
	if (mCommonName->text() != QString())
		return true;
	
	return false;
}

void
X509SubjectDataWidget::clear()
{
	mCountryCode->clear();
	mStateOrProvinceName->clear();
	mLocalityName->clear();
	mOrganizationName->clear();
	mOrganizationalUnitName->clear();
	mCommonName->clear();
	mEmailAddress->clear();
}
