/*
 *  Copyright (C) 2007-2008 CMM Sigma Andrzej Chmielowiec <cmmsigma@cmmsigma.eu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#include "MainWindow.h"
#include "FileDialogWPR.h"
//#include "GenCertReqWidget.h"
#include "QDetailsMessageBox.h"
//#include "AboutBox.h"

#define VERSION "1.3"

extern QString lang;

// MainWindow class
MainWindow::MainWindow()
{
	setWindowTitle(tr("sToken SoftAdmin"));
	setWindowIcon(QIcon(QString(":img/stoken_softadmin_032.png")));
	
	// OpenSSL command and configuretion.
	mOpensslCmd = QCoreApplication::applicationDirPath().append("/../bin/stoken_openssl");
#ifdef Q_WS_WIN
	mOpensslCmd.append(QString(".exe"));
#endif
	mOpensslConfFilePath = QDir::currentPath() + QDir::separator() + "openssl.cnf";
	
	if (!QFile::exists(mOpensslConfFilePath))
	{
		QFile srcFile(":openssl.cnf");
		srcFile.open(QIODevice::ReadOnly);
		QByteArray content = srcFile.readAll();
		srcFile.close();
		
		QFile dstFile(mOpensslConfFilePath);
		dstFile.open(QIODevice::WriteOnly | QIODevice::Truncate);
		dstFile.write(content);
		dstFile.close();
	}
	
	// Window layout.
	QGridLayout *layout;
	
	// Logo.
	QLabel *logo = new QLabel;
	logo->setPixmap(QPixmap(QString(":img/stoken_logo_120_60.png")));
	logo->setAlignment(Qt::AlignRight);
	
	// Private key.
	mKeyPath = new QLineEdit;
	mSelectKey = new QPushButton(tr("Select"));
	connect(mSelectKey, SIGNAL(clicked()), this, SLOT(selectKeySlot()));
	mKeyAlgo = new QComboBox();
	mKeyAlgo->addItem(QString("ECDSA-256"), QVariant(QString("ec:prime256v1")));
	mKeyAlgo->addItem(QString("ECDSA-384"), QVariant(QString("ec:secp384r1")));
	mKeyAlgo->addItem(QString("RSA-1024"), QVariant(QString("rsa:1024")));
	mKeyAlgo->addItem(QString("RSA-2048"), QVariant(QString("rsa:2048")));
	mKeyAlgo->addItem(QString("RSA-3072"), QVariant(QString("rsa:3072")));
	mKeyAlgo->addItem(QString("RSA-4096"), QVariant(QString("rsa:4096")));
	mKeyAlgo->setCurrentIndex(0);
	mKeyMD = new QComboBox();
	mKeyMD->addItem(QString("SHA-256"), QVariant(QString("sha256")));
	mKeyMD->addItem(QString("SHA-224"), QVariant(QString("sha224")));
	mKeyMD->addItem(QString("SHA-1"), QVariant(QString("sha1")));
	mKeyMD->setCurrentIndex(0);
	mKeyEncrypt = new QComboBox();
	mKeyEncrypt->addItem(tr("Yes"));
	mKeyEncrypt->addItem(tr("No"));
	mKeyEncrypt->setCurrentIndex(0);
	
	layout = new QGridLayout;
	layout->addWidget(new QLabel(tr("Key file")), 0, 0);
	layout->addWidget(mKeyPath, 0, 1);
	layout->addWidget(mSelectKey, 0, 2);
	layout->addWidget(new QLabel(tr("Key algorithm")), 1, 0);
	layout->addWidget(mKeyAlgo, 1, 1, 1, 2);
	layout->addWidget(new QLabel(tr("Hash algorithm")), 2, 0);
	layout->addWidget(mKeyMD, 2, 1, 1, 2);
	layout->addWidget(new QLabel(tr("Key encryption")), 3, 0);
	layout->addWidget(mKeyEncrypt, 3, 1, 1, 2);
	layout->setColumnStretch(1, 1);
	layout->setRowStretch(4, 1);
	
	QWidget *keyBox = new QWidget;
	keyBox->setLayout(layout);
	
	// Request.
	mSubjectData = new X509SubjectDataWidget(this);
	
	// Generation button.
	mGenerate = new QPushButton(tr("Generate"));
	connect(mGenerate, SIGNAL(clicked()), this, SLOT(generateCertReqSlot()));
	
	// Key and Request widgets.
	QTabWidget *keyAndReqTab = new QTabWidget;
	keyAndReqTab->addTab(keyBox, tr("Private key"));
	keyAndReqTab->addTab(mSubjectData, tr("Request"));
	
	layout = new QGridLayout;
	layout->addWidget(keyAndReqTab, 0, 0, 1, 2);
	layout->addWidget(mGenerate, 2, 1, 1, 1);
	layout->setColumnStretch(0, 1);
	layout->setRowStretch(1, 1);
	
	QWidget *keyAndReq = new QWidget;
	keyAndReq->setLayout(layout);
	
	// PKCS#12 certificate generation.
	mP12KeyPath = new QLineEdit;
	mP12SelectKey = new QPushButton(tr("Select"));
	connect(mP12SelectKey, SIGNAL(clicked()), this, SLOT(selectP12KeySlot()));
	mP12InCertPath = new QLineEdit;
	mP12SelectInCert = new QPushButton(tr("Select"));
	connect(mP12SelectInCert, SIGNAL(clicked()), this, SLOT(selectP12InCertSlot()));
	mP12OutCertPath = new QLineEdit;
	mP12SelectOutCert = new QPushButton(tr("Select"));
	connect(mP12SelectOutCert, SIGNAL(clicked()), this, SLOT(selectP12OutCertSlot()));
	mP12Generate = new QPushButton(tr("Generate"));
	connect(mP12Generate, SIGNAL(clicked()), this, SLOT(generateP12Slot()));
	
	QGroupBox *p12InBox = new QGroupBox(tr("Input"));
	layout = new QGridLayout;
	layout->addWidget(new QLabel(tr("Key file")), 0, 0);
	layout->addWidget(mP12KeyPath, 0, 1);
	layout->addWidget(mP12SelectKey, 0, 2);
	layout->addWidget(new QLabel(tr("Certificates")), 1, 0);
	layout->addWidget(mP12InCertPath, 1, 1);
	layout->addWidget(mP12SelectInCert, 1, 2);
	layout->setColumnStretch(1, 1);
	p12InBox->setLayout(layout);
	
	QGroupBox *p12OutBox = new QGroupBox(tr("Output"));	
	layout = new QGridLayout;
	layout->addWidget(new QLabel(tr("PKCS#12")), 0, 0);
	layout->addWidget(mP12OutCertPath, 0, 1);
	layout->addWidget(mP12SelectOutCert, 0, 2);
	layout->setColumnStretch(1, 1);
	p12OutBox->setLayout(layout);
	
	layout = new QGridLayout;
	layout->addWidget(p12InBox, 0, 0, 1, 2);
	layout->addWidget(p12OutBox, 1, 0, 1, 2);
	layout->addWidget(mP12Generate, 3, 1);
	layout->setColumnStretch(0, 1);
	layout->setRowStretch(2, 1);
	
	QWidget *pkcs12 = new QWidget;
	pkcs12->setLayout(layout);
	
	// All together.
	QTabWidget *mainTab = new QTabWidget;
	mainTab->addTab(keyAndReq, tr("Key and Request"));
	mainTab->addTab(pkcs12, tr("PKCS#12"));
	
	layout = new QGridLayout;
	layout->addWidget(logo, 0, 0, 1, 2);
	layout->addWidget(mainTab, 1, 0, 1, 2);
	
	QWidget *mainWidget = new QWidget;
	mainWidget->setLayout(layout);
	setCentralWidget(mainWidget);
	
	createActions();
	createMenus();
	createAboutBox();
	
	setMinimumWidth(450);
}

void
MainWindow::selectKeySlot()
{
	QString keyPath = FileDialogWPR::getSaveFileName(this, tr("Save private key as"),
		QString(), tr("Key files (*.key);;All files (*)"));
	
	if (!keyPath.isEmpty())
	{
		mKeyPath->setText(keyPath);
	}
}

void
MainWindow::generateCertReqSlot()
{
	// Check key path.
	if (mKeyPath->text().isEmpty())
	{
		QMessageBox::information(this, tr("Information"), tr("Key file cannot be empty."));
		return;
	}
	
	// Check request.
	if (!mSubjectData->isReady())
	{
		QMessageBox::information(this, tr("Information"), tr("Common name field cannot "
			"be empty."));
		return;
	}
	
	// Get and check password.
	QString pass;
	
	if (mKeyEncrypt->currentText() == tr("Yes"))
	{
		pass = getPass(true);
		
		if (pass.isEmpty())
			return;
	}
	
	// Request file path.
	QString reqPath = mKeyPath->text();
	reqPath.remove(QString(".key"));
	reqPath.append(QString(".req"));
	reqPath = FileDialogWPR::getSaveFileName(this, tr("Save request as"),
		reqPath, tr("Requests (*.req);;All files (*)"));
	
	// Key signature and digest algorithm.
	QString algo = mKeyAlgo->itemData(mKeyAlgo->currentIndex()).toString();
	QString md = mKeyMD->itemData(mKeyMD->currentIndex()).toString();
	
	if (algo.startsWith(QString("ec:")))
	{
		md = QString("ecdsa-with-") + md;
	}
	else
	{
		md = md + QString("WithRSAEncryption");
	}
	
	// Generate ECDSA params set I.
	mOpensslArgs.clear();
	mOpensslArgs << "ecparam" << "-name" << "prime256v1" << "-out" << "prime256v1";
	
	if (!runOpenssl())
	{
		QDetailsMessageBox::critical(this, QObject::tr("Error"), 
			QObject::tr("Cannot create domain file."), mOpensslError);
		return;
	}
	
	// Generate ECDSA params set II.
	mOpensslArgs.clear();
	mOpensslArgs << "ecparam" << "-name" << "secp384r1" << "-out" << "secp384r1";
	
	if (!runOpenssl())
	{
		QDetailsMessageBox::critical(this, QObject::tr("Error"), 
			QObject::tr("Cannot create domain file."), mOpensslError);
		return;
	}
	
	// Run OpenSSL and create request.
	mOpensslArgs.clear();
	mOpensslArgs << "req" << "-new" << "-newkey" << algo << "-keyout" << mKeyPath->text() <<
		"-md" << md << "-utf8" << "-batch" << "-config" << mOpensslConfFilePath <<
		"-subj" << mSubjectData->subjectOpenSSL() << "-out" << reqPath;
	
	if (mKeyEncrypt->currentText() == tr("Yes"))
	{
		mOpensslArgs << "-passout" << QString("pass:") + pass;
	}
	else
	{
		mOpensslArgs << "-nodes";
	}
	
	if (!runOpenssl())
	{
		if(mOpensslError.contains("error:0906A065"))
		{
			QDetailsMessageBox::critical(this, tr("Error"), tr("Incorrect password."),
				mOpensslError);
		}
		else
		{
			QDetailsMessageBox::critical(this, tr("Error"), tr("Cannot create request."), 
				mOpensslError);
		}
	}
	else
	{
		QMessageBox::information(this, tr("Information"), tr("Request was created successfully."));
	}
	
	mKeyPath->clear();
	mSubjectData->clear();
}

void
MainWindow::selectP12KeySlot()
{
	QString keyPath = FileDialogWPR::getOpenFileName(this, tr("Select private key"),
		QString(), tr("Key files (*.key);;All files (*)"));
	
	if (!keyPath.isEmpty())
	{
		mP12KeyPath->setText(keyPath);
	}
}

void
MainWindow::selectP12InCertSlot()
{
	QString certPath = FileDialogWPR::getOpenFileNames(this, tr("Select certificates"),
		QString(), tr("Certificates (*.pem *.cert *.crt);;All files (*)")).join(";");
	
	if (!certPath.isEmpty())
	{
		mP12InCertPath->setText(certPath);
	}
}

void
MainWindow::selectP12OutCertSlot()
{
	QString certPath = FileDialogWPR::getSaveFileName(this, tr("Save PKCS#12 certificate as"),
		QString(), tr("Certificates (*.p12);;All files (*)"));
	
	if (!certPath.isEmpty())
	{
		mP12OutCertPath->setText(certPath);
	}
}

void
MainWindow::generateP12Slot()
{
	QStringList filePaths = mP12InCertPath->text().split(";");
	QByteArray allData;
	
	filePaths.prepend(mP12KeyPath->text());
	
	for (int i = 0; i < filePaths.count(); i++) {
		QFile file(filePaths[i]);
		if (!file.open(QIODevice::ReadOnly)) {
			QMessageBox::critical(this, tr("Error"), tr("Could not open file %1 "
				"to read.").arg(file.fileName()));
			return;
		}
		allData += file.readAll();
		file.close();
	}
	
	bool keyIsEncrypted = (allData.indexOf(QString("ENCRYPTED")) != -1);
	
	QDir tempDir = QDir::temp();
	QFile tempFile;
	
	for (int i = 1; i != 0; i++) {
		if (!tempDir.exists(QString("%1").arg(i))) {
			tempFile.setFileName(QDir::tempPath() + QDir::separator() + QString("%1").arg(i));
			if (!tempFile.open(QIODevice::ReadWrite)) {
				QMessageBox::critical(this, tr("Error"), tr("Could not open file %1 "
					"to read and write.").arg(tempFile.fileName()));
				return;
			}
			tempFile.write(allData);
			tempFile.close();
			break;
		}
	}
	
	// Get and check password.
	QString pass = getPass(!keyIsEncrypted);
		
	if (pass.isEmpty())
		return;
	
	// Run OpenSSL.
	mOpensslArgs.clear();
	mOpensslArgs << "pkcs12" << "-export" << "-in" << tempFile.fileName() << "-out" << 
		mP12OutCertPath->text() << "-passout" << "pass:" + pass;
	
	if (keyIsEncrypted) {
		mOpensslArgs << "-passin" << "pass:" + pass;
	}
	
	if (!runOpenssl()) {
		QDetailsMessageBox::critical(this, tr("Error"), tr("Cannot create PKCS#12 "
			"certificate."), mOpensslError);
	}
	else {
		QMessageBox::information(this, tr("Information"), tr("PKCS#12 certificate "
			"was generated successfully."));
	}
	
	mP12KeyPath->clear();
	mP12InCertPath->clear();
	mP12OutCertPath->clear();
	
	QFile::remove(tempFile.fileName());
}

void
MainWindow::helpSlot()
{
	mAboutBox->setActiveHelp();
	mAboutBox->exec();
}

void
MainWindow::aboutSlot()
{
	mAboutBox->setActiveAbout();
	mAboutBox->exec();
}

void
MainWindow::langEnSlot()
{
	lang = "";
	langExclamation();
}

void
MainWindow::langPlSlot()
{
	lang = "pl";
	langExclamation();
}

QString
MainWindow::getPass(bool confirm)
{
	bool ok;
	QString pass;
	QString repass;
	
	do
	{
		pass = QInputDialog::getText(this, tr("Enter private key password"),
			tr("Password:"), QLineEdit::Password, QString(), &ok);
		
		if (!ok)
			return QString();
		
		if (pass.isEmpty())
			QMessageBox::warning(this, tr("Information"), tr("Key password cannot be empty."));
	}
	while (pass.isEmpty());
	
	if (confirm)
	{
		repass = QInputDialog::getText(this, tr("Re-enter password"),
			tr("Re-entered password:"), QLineEdit::Password, QString(), &ok);
		
		if (!ok)
			return QString();
		
		if(pass != repass)
		{
			QMessageBox::information(this, tr("Information"), tr("Entered passwords are not equal."));
			return QString();
		}
		else
			return pass;
	}
	else
		return pass;
}

void
MainWindow::langExclamation()
{
	QMessageBox::information(this, tr("Information"), tr("Application language was "
		"changed. Please restart application to apply changes."));
}

bool
MainWindow::runOpenssl()
{
	QProcess openssl;
	
	openssl.setWorkingDirectory(QDir::homePath() + QDir::separator() +
		QString(".stoken_softadmin"));
	openssl.start(mOpensslCmd, mOpensslArgs);
	
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	
	while (openssl.state() != QProcess::NotRunning)
		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
	
	QApplication::restoreOverrideCursor();
	
	mOpensslOut.clear();
	mOpensslError.clear();
	
	mOpensslOut = QString(openssl.readAllStandardOutput());
	mOpensslError = QString(openssl.readAllStandardError());
	
	if (openssl.exitCode() != 0)
		return false;
	
	return true;
}

void
MainWindow::createActions()
{
	mHelpAct = new QAction(QIcon(QString(":img/contents_ii.png")), tr("Manual"), this);
	mHelpAct->setShortcut(tr("Ctrl+H"));
	connect(mHelpAct, SIGNAL(triggered()), this, SLOT(helpSlot()));
	
	mAboutAct = new QAction(QIcon(QString(":img/info.png")), tr("About"), this);
	connect(mAboutAct, SIGNAL(triggered()), this, SLOT(aboutSlot()));
	
	mExitAct = new QAction(QIcon(QString(":img/exit.png")), tr("Exit"), this);
	mExitAct->setShortcut(tr("Ctrl+Q"));
	connect(mExitAct, SIGNAL(triggered()), this, SLOT(close()));
	
	mLangEn = new QAction(tr("English"), this);
	mLangEn->setCheckable(true);
	connect(mLangEn, SIGNAL(triggered()), this, SLOT(langEnSlot()));
	mLangPl = new QAction(tr("Polish"), this);
	mLangPl->setCheckable(true);
	connect(mLangPl, SIGNAL(triggered()), this, SLOT(langPlSlot()));
	
	mLang = new QActionGroup(this);
	mLang->addAction(mLangEn);
	mLang->addAction(mLangPl);
}

void
MainWindow::createMenus()
{
	QMenu *menu, *language;
	
	menu = menuBar()->addMenu(tr("&General"));
	
	language = menu->addMenu(tr("&Language"));
	language->addAction(mLangEn);
	language->addAction(mLangPl);
	
	if (lang == "pl")
		mLangPl->setChecked(true);
	else
		mLangEn->setChecked(true);
	
	menu->addSeparator();
	
	menu->addAction(mExitAct);
	
	menu = menuBar()->addMenu(tr("&Help"));
	menu->addAction(mHelpAct);
	menu->addSeparator();
	menu->addAction(mAboutAct);
}

void
MainWindow::createToolBars()
{
}

void
MainWindow::createAboutBox()
{
	mAboutBox = new AboutBox(this);
	
	// About informations.
	mAboutBox->setAboutPixmap(QPixmap(QString(":img/stoken_about_softadmin_384.png")));
	mAboutBox->setAboutInfo(tr("<h2>sToken SoftAdmin " VERSION "</h2>"
	"<br>This program is free software: you can redistribute it and/or modify "
	"it under the terms of the GNU General Public License as published by the Free "
	"Software Foundation, either version 3 of the License, or (at your option) any "
	"later version.<br><br>"
	"<a href=\"http://www.stoken.eu/soft/index.en.html\">http://www.stoken.eu/soft</a>"));
	
	// Help informations.
	mAboutBox->setHelpPixmap(QPixmap(QString(":img/stoken_about_softadmin_384.png")));
	mAboutBox->setHelpInfo(tr("<h2>sToken SoftAdmin v" VERSION "</h2>"
	"<br>Online instructions to this application are avaliable on the following "
	"website: <a href=\"http://www.stoken.eu/products/manuals.en.html\">"
	"http://www.stoken.eu</a>"));
	
	// License.
	QFile file(QString(":LICENSE"));
	file.open(QIODevice::ReadOnly);
	QByteArray license = file.readAll();
	file.close();
	mAboutBox->setLicense(QString(license.data()));
	
	// Thanks.
	mAboutBox->setThanksPixmap(QPixmap(QString(":img/stoken_about_softadmin_384.png")));
	mAboutBox->setThanksInfo(tr("<h4>Thanks to</h4>"
		"Andrzej Chmielowiec, Wiktor Kazana"
		"<h4>Special thanks</h4>"
		"Special thanks to all people who participate in development process of "
		"<a href=\"www.openssl.org\">OpenSSL</a> library and "
		"<a href=\"www.openvpn.net\">OpenVPN</a> software. Both of this projects "
		"are crucial for this application."
		));
}
